/*
------ CONTROLLEUR DE RENARDS Montreal fox controller -----

Auteur:	VE2EMM										Date: Jan 2003

------------------  Details du projet  -------------------

PIC16F628 Xtal = 4.194304 mHz

	Port A
		RA0 = LED & SON
		RA1 = OOK CW
		RA2 = TX on
		RA3 = BP load a letter & jumper
		RA4 = S1 DIP9

	Port B
		RB0 = DIP1
		RB1 = DIP2
		RB2 = DIP3
		RB3 = DIP4
		RB4 = DIP5
		RB5 = DIP6
		RB6 = DIP7
		RB7 = DIP8

	-------------------------------------

	DIT = 1 period
	DAH = 3 periods
	SPACE between character = 1 periods
	SPace between letters = 3 periods
	SPace between words = 7 periods

----------------------------------------------------------------
*/

#include <16F628.h>

#fuses xt,nowdt,noprotect,put,NOCPD,NOLVP,MCLR,BROWNOUT
#use delay (clock=4194304)
#use fast_io(A)
#use fast_io(B)


#define son pin_A0		//sound
#define led pin_A0
#define ook_cw pin_A1	//80 meter keying
#define tx_key pin_A2	//tx on
#define bp_jp pin_A3	//load a letter button
							//charger la EEDATA	avec VE2EMM
#rom 0x2100={0x18,0x02,0x3c,0x02,0x07,0x07,0x29,0x3c,0xff}

//----------------- declaration des variables ------------------

int1 flag_sec=0;
int1 flag_debut_min=0;
int1 flag_led=0;
int1 flag_ook_cw=0;

int8 i=0;// memoires temporaires
int8 j=0;
int8 k=0;
int8 cmpt_int=16;
int8 secondes=0;
int8 min_unit=0;
int8 min_diz=0;
int8 lettre_cw=0;
int8 vitesse_cw=0;//vitesse du CW en WPM
int16 num_periods=0;
int8 temps_att_diz=0; // temps d'attente en 10 minutes
int8 call_id=0;//ID du fox
int8 nomb_fox=0;//nombre de fox dans la chasse
int8 min_cycle=1;
int8 tmpw;// memoires pour sauver W et Status
int8 tmps;

//--------------------- constants ---------

int8 const id[24]={0x07,0x0f,0x00,0x07,0x0f,0x02,0x07,0x0f,0x04,
	0x07,0x0f,0x08,0x07,0x0f,0x10,0x07,0x0f,0x20,0x07,0x0f,0x05,
	0x07,0x0f,0x09};	// MO0, MOE, MOI, MOS, MOH, MO5, MON, MOD

int8 const de[2]={0x09,0x02};	// DE

//----------------- Declaration des fonctions ------------------

void pic_ini(void);
void lire_dip(void);
void chasse_ini(void);
void temps_attente(void);
void chasser(void);// faire la chasse

void chronometrage(void);//isr de temps
void timing(void);// suite du temps

void memoriser_lettres(void);//memoriser les lettres
void une_lettre(char lettre_cw);
void tone(int8 nom_dit);//generer tones
void no_tone(int8 nom_dit);//generer temps morts

//****************************** Programme principal ********************

main()
{
	pic_ini();
	if(input(bp_jp))memoriser_lettres();//si cavalier absent - read jumper
	else
	{
		lire_dip();        // read dip switch
		chasse_ini();      // set hunt parameters
		temps_attente();   // wait for startup time
		chasser();         //jouer au renard, chasser - hunt
	}
}

//************************* Fin du programme principal *******************

void pic_ini(void)//----------------------------
{
   setup_vref(FALSE);
   setup_comparator(FALSE);
   SETUP_TIMER_1(T1_DISABLED);

	output_A(0x00);
	output_B(0x00);
	set_tris_A(0b00011000);// 0-2 en sortie, 3-4 en entree
	set_tris_B(0b11111111);// tout en entree
	port_b_pullups(true);
}

lire_dip()//------------------- lire les dip switches --------
{
	k=input_b();
                   //temps attente avant la chasse en 10 minutes
	temps_att_diz=(k&0b00000111)*3;
                   //--- call id du fox
	call_id=(k&0b00111000);
	shift_right(&call_id,1,0);
	shift_right(&call_id,1,0);
	shift_right(&call_id,1,0);
                    // nombre de foxes dans la chasse
	shift_left(&nomb_fox,1,input(pin_a4));
	shift_left(&nomb_fox,1,input(pin_b7));
	shift_left(&nomb_fox,1,input(pin_b6));
                     // trouble, id du fox > nombre foxes
	while(call_id>nomb_fox)
	{
		output_high(led);
		delay_ms(127);
		output_low(led);
		delay_ms(127);
	}
}

chasse_ini()//---------------------------------------
{
	cmpt_int=16;
	secondes=0;
	min_unit=0;
	min_diz=temps_att_diz;
	min_cycle=1;
	set_rtcc(0);
	setup_counters(rtcc_internal,rtcc_div_256);
	enable_interrupts(INT_RTCC);
	enable_interrupts(global);
}

temps_attente()//--------------------- Attente avant la chasse
{

	if(min_diz>0)
	{
      do
      {
         timing();

         if(flag_sec==1)
		  {
		 	  flag_led=!flag_led;
			  output_bit(led,flag_led);
			  flag_sec=0;
		  }
      }
      while(min_diz>0);
	}
	flag_debut_min=1;
}

void chasser(void)//---------------- chasser ---------------------
{
	flag_ook_cw=1;
	while(true)
	{
		timing();
		while(((min_cycle==call_id)||(call_id==0))&&flag_debut_min==1)
		{
			output_high(tx_key);	//0 sec
			flag_debut_min=0;
			vitesse_cw=7;
			while(secondes<1);		// 1 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<7);		// 7 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<13);		// 13 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<19);		// 19 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<25);		// 25 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<31);		// 31 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<37);		// 37 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<43);		// 43 sec
				for(i=0;i<=2;i++)une_lettre(id[(call_id*3)+i]);
			while(secondes<49);		// 49 sec
				vitesse_cw=20;
				for(i=0;i<=1;i++)une_lettre(de[i]);//envoyer DE
				no_tone(7);//espace
				i=0;
				while(read_eeprom(i)!=0xff)
				{
					une_lettre(read_eeprom(i));
					i++;
				}
			while(secondes<59);		// 59 sec
			output_low(tx_key);		// arret de tx
		}
	}
}

//***********************************************************************

#int_global//-------- fonction d'interruption ------------------
chronometrage(void)
{
#asm 					//-------------------------- ** PUSH **
	MOVWF	TMPW		//SAUVER W DANS TMPW
	SWAPF	0x03,W		//Charger le STATUS dans W
	MOVWF	TMPS		//Sauver le STATUS dans TMPS
#endasm
	if(--cmpt_int==0)
	{
		cmpt_int=16;//16
		++secondes;//max secondes=60;
		flag_sec=1;
	}
					//---------------------------- ** POP **
#asm
	BCF		0x0B,2		//Rappel du flag du zero de TMRO
	SWAPF	TMPS,W		//replacer W, STATUS
	MOVWF	0x03
	SWAPF	TMPW,F		//replacer STATUS
	SWAPF	TMPW,W
	RETFIE				//Return to the program
#endasm
}

//--------------------- autres Fonctions -----------------------

timing(void)//- finir chronometrage a l'exterieur de l'interruption --
{
		if(secondes>=60)
	{
		secondes=0;
		++min_unit;

		++min_cycle;
		if(min_cycle>nomb_fox)min_cycle=1;
		flag_debut_min=1;
	}
	if(min_unit>=10)
	{
		min_unit=0;
		if(min_diz>0)min_diz--;
	}
}

memoriser_lettres()//--- Memoriser les lettres d'appel - LOAD CALL LETTERS
{
	vitesse_cw=5;//vitesse du CW 5 WPM
	for(i=0;i<=1;i++)une_lettre(de[i]);//envoyer DE
	i=0;
	do
	{
		while(input(bp_jp)==1);//attente du BP entree
		k=input_b();
		write_eeprom(i,k);
		une_lettre(read_eeprom(i));//confirmer l'enregistrement
		i++;
	}
	while(k!=0xff);
	sleep();
}

//************************ generation CW ******************

une_lettre(char lettre_cw)//------ faire un caractere -----------
{
	if(lettre_cw==0)no_tone(5);
	while(lettre_cw>1)
	{
		if(shift_right(&lettre_cw,1,0))//dah
		{
			tone(3);
			no_tone(1);
		}
		else							//dit
		{
			tone(1);
			no_tone(1);
		}
	}

	no_tone(2);
}

void tone(int8 nom_dit)//------------ dits --------------
{
	num_periods = (1000/vitesse_cw)*nom_dit;//vitesse en wpm
	if(flag_ook_cw)output_high(ook_cw);
	while((num_periods--) != 0)
	{
		delay_us(255);
		delay_us(255);
		output_high(son);//demi cycle positif
		delay_us(255);
		delay_us(255);
		output_low(son);//demi cycle negatif
	}
	output_low(ook_cw);
}

void no_tone(int8 nom_dit)//---------- espaces -------------
{
	num_periods = (1000/vitesse_cw)*nom_dit;//vitesse en wpm
	while((num_periods--) != 0)
	{
		delay_us(255);
		delay_us(255);
		delay_cycles(1);
		delay_us(255);
		delay_us(255);
		delay_cycles(1);
	}
}

//************************ fin generation CW ******************

//----------------------- fin du programme ----------------------

/*
=== SNIP ==================================================================

 MORSE ENCODING ...

 One morse character per BYTE, bitwise, LSB to MSB.

 0 = dit, 1 = dah.  The byte is shifted to the right bit by bit, until the
 last 1 is left, this 1 is an END OF CHARACTER indicator.
 A maximum of 7 elements can be encoded, (error) is excluded.

                        SWITCHES
     CODE               MSD  LSD    Comments
    ------              ---  ---    --------------------
 KN -.--.   00101101	    2   13  	Go only
 SK ...-.-  01101000  	 5    8	   Clear
 AR .-.-.   00101010  	 2   10	   Over, end of message
 BT -...-   00110001  	 3    1	   Pause
 AS .-...   00100010	    2    2	   Wait, stand by
  / -..-.   00101001  	 2    9
  0 -----   00111111  	 3   15
  1 .----   00111110  	 3   14
  2 ..---   00111100  	 3   12
  3 ...--   00111000  	 3    8
  4 ....-   00110000  	 3    0
  5 .....   00100000  	 2    0
  6 -....   00100001  	 2    1
  7 --...   00100011  	 2    3
  8 ---..   00100111  	 2    7
  9 ----.   00101111  	 2   15
  A .-      00000110  	 0    6
  B -...    00010001  	 1    1
  C -.-.    00010101  	 1    5
  D -..     00001001  	 0    9
  E .       00000010  	 0    2
  F ..-.    00010100  	 1    4
  G --.     00001011  	 0   11
  H ....    00010000  	 1    0
  I ..      00000100  	 0    4
  J .---    00011110  	 1   14
  K -.-     00001101  	 0   13
  L .-..    00010010  	 1    2
  M --      00000111  	 0    7
  N -.      00000101  	 0    5
  O ---     00001111  	 0   15
  P .--.    00010110  	 1    6
  Q --.-    00011011  	 1   11
  R .-.     00001010  	 0   10
  S ...     00001000  	 0    8
  T -       00000011  	 0    3
  U ..-     00001100  	 0   12
  V ...-    00011000  	 1    8
  W .--     00001110  	 0   14
  X -..-    00011001  	 1    9
  Y -.--    00011101  	 1   13
  Z --..    00010011  	 1    3
  SPACE	   00000000  	 0    0 	  Word space (special exception)
  EOM       11111111  	15   15	  End of message  (special exception)*****

=== SNIP ==================================================================
*/
